/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

/* Software by P.J.Sloetjes. Version 1.7. */

"use strict";

const {classes: Cc, interfaces: Ci, utils: Cu} = Components;
Cu.import("resource://gre/modules/Services.jsm");
const Sv = Services;
const Rp = Sv.io.getProtocolHandler("resource")
		.QueryInterface(Ci.nsIResProtocolHandler);

const
	dc = "document",
	dv = "defaultView",
	gb = "gBrowser",
	nb = "navigator:browser",
	nt = "navigator-toolbox",
	od = "ownerDocument",
	ot = "originalTarget",
	pn = "parentNode",
	tg = "target",
	ud,
	xh = "http://www.w3.org/1999/xhtml";

var _$ = null,
	_B = "undefined";

function addonInit(p, r) {}
function addonExit(p, r) {}
function addonMenu(d, b) {}
function addonWake(d, e, b) {}

function install() {}

function uninstall(e, r) {
	if (r !== ADDON_UNINSTALL)
		return;
	Sv.prefs.deleteBranch(
		"extensions." + e.id + ".");
}

function startup(e, r) {
	const i = e.id.split("@")[0],
		p = "resource://" + i + "/";
	_B = "extensions." + e.id + ".";
	Rp.setSubstitution(i, 
		e.resourceURI);
	Sv.scriptloader
		.loadSubScript(
			p + i + ".js");
	modules(1, p, mo);
	load$();
	addonInit(p, r);
	if (r === ADDON_INSTALL)
		for(e in pr)
			pref(e);
	installInAll(1, r);
}

function shutdown(e, r) {
	const i = e.id.split("@")[0],
		p = "resource://" + i + "/";
	installInAll(0);
	addonExit(p, r);
	modules(0, p, mo);
	Rp.setSubstitution(i, null);
	_$ = null;
	Sv.strings.flushBundles();
}

function modules(i, r, a) {
	r = r || "";
	a && a.forEach && a.forEach(
		function(s) {
			i ? Cu.import(r + s) :
				Cu.unload(r + s);
		} );
}

function load$() {
	var c, i;
	Sv.strings.flushBundles();
	c = Cc["@mozilla.org/chrome/chrome-registry;1"]
		.getService(Ci.nsIXULChromeRegistry)
			.getSelectedLocale("global");
	i = 3;
	while (i-- && !$("eid")) {
		_$ = Sv.strings.createBundle(
			"resource://" + id + "/_" +
			(i ? c : "en-US") + ".txt");
		(i === 2) && (c = c.substring(0, 2))
	}
}

const onOpenWin = {
	observe : function (e) {
		on(1, e, { 
			load: function onLoad(e) {
				on(0, e[tg][dv], {
					load: onLoad } );
				installGUI(e[tg], 1)
			}
		});
	}
}

const onCloseWin = {
	observe : function (e) {
		installGUI(e[dc], 0);
	}
}

function installInAll(e, r) {
	browsers(installGUI, e, r);
	r = e ? Sv.ww.registerNotification :
			Sv.ww.unregisterNotification;
	r(onCloseWin);
	r(onOpenWin);
}

function installGUI(d, e, r) {
	var b;
	if (!el(d, "nav-bar"))
		return;
	!e && (b = el(d, nt).palette) && 
		(b = ela(b, id));
	b = b || el(d, id);
	if (e && !b) {
		b = createButton(d);
		placeButton(d, b, r);
		addonMenu(d, b);
	}
	on(e, d[dv], {
		beforecustomization: startCustomize,
		aftercustomization: endCustomize });
	addonWake(d, e, b);
	if (!e && b && (e = b[pn])) {
		e.removeChild(b);
		b = e.currentSet;
		if (b && (r === ADDON_UNINSTALL)) {
			e.setAttribute("currentset", b);
			d.persist(e.id, "currentset");
		}
	}
}

function createButton(d) {
	return gui(d, "toolbarbutton", "", 
		"list-style-image:" + 
			"url('resource://" + 
			id + "/icon16.png');" +
			"-moz-image-region:" + 
			icon(0) + ";",
		{"class": "toolbarbutton-1",
			type: "menu-button",
			title: $("eid"),
			tooltiptext: $("btt")});
}

function placeButton(d, b, r) {
	const t = el(d, nt),
		v = el(d, "nav-bar");
	var e, s, c, n, a, i;
	if (!t || !v)
		return;
	t.palette.appendChild(b);
	a = insertButton();
	if (!a && (r === ADDON_INSTALL || 
			r === ADDON_ENABLE)) {
		appendButton();
		insertButton()
	}
	function appendButton() {
		if (!(s = v.currentSet))
			return;
		v.currentSet = (s += "," + id);
		v.setAttribute("currentset", s);
		d.persist(v.id, "currentset");
	}
	function insertButton() {
		b = eln(d, "toolbar");
		i = 0;
		while ((e = b[i++]))
			if ((s = e.getAttribute("currentset").split(",")) &&
					((n = s.indexOf(id)) >= 0)) {
				c = e.firstChild;
				while (c && (s.indexOf(c.id) < n))
					c = c.nextSibling;
				return e.insertItem(id, c);
			}
	}
}

function startCustomize(e) {
	browsers(addonWake, 0);
}

function endCustomize(e) {
	browsers(mirrorCustomize, e[tg][od]);
	browsers(addonWake, 1)
}

function mirrorCustomize(d, b) {
	if (!(e = el(d, nt).palette))
		return;
	var b = el(b, id), p, i, c, e;
	if (!b && (b = el(d, id)))
		e.appendChild(b[pn]
			.removeChild(b));
	else if (b && (p = b[pn]) && 
			(e = ela(e, id) || el(d, id)) && 
			e[pn] && (i = Array.indexOf(
				p.childNodes, b) + 1) &&
			(b = el(d, p.id)) &&
			(c = b.childNodes)) {
		e = e[pn].removeChild(e);
		i > c.length ?
			b.appendChild(e) :
			b.insertBefore(e, c[i - 1]);
	}
}



function $(e) {
	try { e = _$.GetStringFromName(e) }
	catch (c) { e = "" }
	return e
}

function browsers(f) {
	var c, a = Array.slice(arguments),
		n = Sv.wm.getEnumerator(nb);
	while (c = n.getNext())
		(a[0] = c[dc]) && 
				(!a[1] || a[1] !== a[0]) &&
			f.apply(this, a);
}

function fromButton(e) {
	return (e = e && e[ot]) &&
		e.getAttribute("anonid") === "button";
}

function confirm(s) {
	return Cc["@mozilla.org/embedcomp/prompt-service;1"]
		.getService(Ci.nsIPromptService)
		.confirm(null, $("eid"), s)
}

function el(d, i) {
	return d && d.getElementById(i);
}

function ela(e, i) {
	return e.getElementsByAttribute("id", i)[0]
}

function eln(e, n) {
	return e && e.getElementsByTagName && 
		e.getElementsByTagName(n || "*");
}

function gui(p, e, i, s, a) {
	e = e !== "canvas" ? 
		(p[od] || p).createElement(e) : 
		(p[od] || p).createElementNS(xh, "canvas");
	i !== ud && e.setAttribute("id", id + i);
	s && e.setAttribute("style",
		s.replace(/;/g, " !important;"));
	for (i in a)
		e.setAttribute(i, a[i]);
	p[od] && p.appendChild(e);
	return e
}

function icon(i, k, s) {
	s = s || 16;
	k = k || 256;
	var j = (i % k) * s;
	i = ((i / k) |0) * s;
	return "rect(" + i + "px, " + 
		(j + s) + "px, " + 
		(i + s) + "px, " + j + "px)";
}

function log(e, a) {
	if (e === ud) {
		e = ""; 
		a = Components.stack;
		while (a = a.caller)
			e = (a.name || "") + "^" + e
	}
	Sv.console.logStringMessage(
		"[" + id.toUpperCase() + "]  " + e)
}

function on(a, e, c) {
	if (e)
		for (var i in c)
			a ? e.addEventListener(i, c[i], 1) : 
				e.removeEventListener(i, c[i], 1)
}

function pref(a, e) {
	var r, p, i = pr[a];
	if (!i)
		return ud;
	p = Sv.prefs.getBranch(_B);
	if (e === ud)  try {
		r = i[0] ? i[0] < 2 ? 
			p.getIntPref(a) : 
			p.getCharPref(a) : 
			p.getBoolPref(a)
	}
	catch (q) {
		e = r = i[1]
	}
	if (e !== ud)
		i[0] ? i[0] < 2 ? 
			p.setIntPref(a, +e) : 
			p.setCharPref(a, e) : 
			p.setBoolPref(a, !!e)
	return r
}
